(*
Copyright (C) 2003-2004 Evgenii Rudnyi, http://Evgenii.Rudnyi.Ru

This software is a copyrighted work licensed under the terms, described
in the file "FREE_LICENSE". 
*)

(* 
	This function estimates a film coefficient, Eq (1), 
	by means of Eqs (2) and (3).
	
	See Specification Manual for details.
	
	Input:
	
		d - diameter of the chamber in m
		v - velocity in m/s
		mu - gas viscosity in kg/(m s)
		cp - specicif heat capacity in J/(K kg) = m^2/(s^2 K)
		kappa - gas heat conductivity in W/(m K) = kg m/(s^3 K)
		ro - density in kg/m^3
	
	Output:
	
		h - film coefficient in W/(m^2 K) = kg/(s^3 K)
		
	The use of another dimension units is possible but this should be
	consistent. For example, if all values are entered in 
	mu_MSKV units, then the film coefficient will be in pW/(mu_m^2 K)

*)

FilmCoef[d_, v_, mu_, cp_, kappa_, ro_] := Module[
	{
		Re = d*v*ro/mu,
		Pr = mu*cp/kappa,
		Nu
	},
	Nu = 0.026*Re^0.8*Pr^0.4;
	Nu*kappa/d
]
