(*
Copyright (C) 2003-2004 Evgenii Rudnyi, http://Evgenii.Rudnyi.Ru

This software is a copyrighted work licensed under the terms, described
in the file "FREE_LICENSE". 
*)

(* for MOR *)

makeMOR[ID_] := Module[{ret},
	If[Length[Select[Symbol["allowedRunMOR"], StringMatchQ[Pyros2`tmp`IP, #] &]] =!= 0,
		Put["modred", ID<>"/refresh.m"];
		ret = Run["../bin/mor4ansys.cmd "<>ID<>" mor.log \"-o red.mor -v -N "<>Symbol["$$N"]<>" -T "<>ToString[Get[ID<>"/tini.txt"]]<>" -C ../nodes.txt -D dirichlet.txt file.emat\""];
		If[ret != 0, Return["There were problems to run mor4ansys: "<>ToString[ret]<>"."]];
		MSPInclude["pyros2/common/Lock"]
	,
		"<p>You are not allowed to run mor4ansys</p>"
	]
];

readAnsysResults[name_String] := Module[{in, ntime, time, ansys, node},
	in = OpenRead[name];
	ntime = Round[Read[in, Number]];
	time = ReadList[in, Number, ntime];
	ansys = {};
	While[
    (node = Read[in, Word, WordSeparators->{" ", "\t", "\n", FromCharacterCode[10], FromCharacterCode[13]}]) =!= EndOfFile,
    AppendTo[ansys, {node, ReadList[in, Number, ntime]}]
  ];
	Close[in];
	{time, ansys}
];

readMOR[name_String, V_Symbol] := Module[
	{in, dimMax, Tini, redC, redK, ninp, funinp, bredin, nout, node, com},
	in = OpenRead[name, DOSTextFormat->True];
	dimMax = Round[Read[in, Number]];
	Tini = Read[in, Number];
	redC = Map[ReadList[in, Number, dimMax] &, Range[dimMax]];
	redK = Map[ReadList[in, Number, dimMax] &, Range[dimMax]];
	ninp = Round[Read[in, Number]];
	funinp = Table[1., {ninp}];
	bredin = Map[ReadList[in, Number, dimMax] &, Range[ninp]];
	nout = Round[Read[in, Number]];
	Scan[
		(com = Read[in, Word, WordSeparators->{" ", "\t", "\n", FromCharacterCode[10], FromCharacterCode[13]}]; 
		V[com] = ReadList[in, Number, dimMax])&, 
		Range[nout]
	];
	Close[in];
	{Tini, redC, redK, Transpose[bredin], funinp}
];

solveMOR[dim_Integer, redC_List, redK_List, bRed_List, inpFun_List, toTime_ /; NumericQ[toTime]] := Module[
	{time, Ared, Cinv, bred, vecT, vecNameT, vecdT, rs, eqs, iniT},
	If[dim > Length[redC], 
		Print["Required dimension is too big."]; 
		Return[]
	];
	time = Timing[
		Cinv = Inverse[Take[redC, dim, dim]];
		Ared = Cinv.Take[-redK, dim, dim];
		bred = Cinv.(Simplify[Take[bRed, dim].inpFun]);
		vecT = Table[ToExpression[StringJoin["tem", ToString[i]]][tau], {i, dim}];
		vecNameT = Table[ToExpression[StringJoin["tem", ToString[i]]], {i, dim}];
		vecdT = Table[
  	  D[ToExpression[StringJoin["tem", ToString[i]]][tau], tau], {i, dim}];
		rs = Ared.vecT + Simplify[bred];
		eqs = Table[vecdT[[i]] == rs[[i]], {i, dim}];
(*		rs = vecT + Flatten[Take[bRed, dim]];
		ls = Take[H, dim, dim].vecdT;
		eqs = Table[ls[[i]] == rs[[i]], {i, dim}];*)
		iniT = Table[
    	ToExpression[StringJoin["tem", ToString[i]]][0] == 0., {i, dim}];
	];
	time = Timing[
		sol = NDSolve[Flatten[{eqs, iniT}], vecNameT, {tau, 0, toTime}, MaxSteps -> 20000, SolveDelayed->False];
	];
	{sol, vecNameT}
];

evaluateMORAt[time_List, sol_List, vecNameT_List] := Module[{evalList, l},
	evalList[list_, time2_] := Map[#[time2] &, list];
	l = vecNameT /. sol[[1]];
	Map[evalList[l, #] &, time]
];

solveMORAt[time_List, dim_Integer, redC_List, redK_List, bRed_List, inpFun_List] := Module[
	{sol, name},
	{sol, name} = solveMOR[dim, redC, redK, bRed, inpFun, time[[Length[time]]]];
	evaluateMORAt[time, sol, name]
];

computeMORNode[n_String, solAt_List, V_Symbol, Tini_ /; NumericQ[Tini]] := Module[{},
	If[!ValueQ[V[n]],
		out = out<>"<p>ERROR: No results for the node "<>n<>"</p>";
		Return[{}];
	];
 	solAt.Take[V[n], Length[solAt[[1]]]] + Tini
];

plotResult[time_List, res_List, title_String] := Module[{i = 0},
	out = out<>MSPShow[Show[
		Map[
			If[Length[time] === Length[#[[1]]],
				ListPlot[Transpose[{time, #[[1]]}], 
					PlotJoined -> True, PlotRange -> All, 
  				PlotStyle -> #[[2]], DisplayFunction -> Identity, Evaluate[plotRules]]
			]&
		,res],
	DisplayFunction -> $DisplayFunction, 
  PlotLabel -> title]]
];

plotAll[time_List, ansys_List, solAt_List, V_Symbol, Tini_ /; NumericQ[Tini]] := Module[{r},
	Scan[(
    r = computeMORNode[#[[1]], solAt, V, Tini];
		If [r != {},
			out = out<>"<p>";
  	  plotResult[time, {{#[[2]], RGBColor[1, 0, 0]}, {r, RGBColor[0, 1, 0]}}, 
        #[[1]] <> " on time"];
    	plotResult[time, {{#[[2]] - r, RGBColor[0, 1, 0]}}, 
        "Difference for " <> #[[1]] <> " on time"];
			out = out<>"</p>"
		]
    ) &, ansys]
];


compareMOR[ID_] := Module[{dim, time, ansys, Tini, redC, redK, redB, inpFun, solAt},
 	dim = Round[MSPToExpression[Symbol["$$N"]]];
	If[!NumericQ[dim], Return["The dimension of the trial reduced model is not a number:"<>Symbol["$$N"]]];
	If[Length[FileNames["results.txt", {ID}]] == 0, Return["There are no ANSYS results in the text form. Please generate them at the Status page"]];
	{time, ansys} = readAnsysResults[ID<>"/results.txt"];
	If[Length[FileNames["red.mor", {ID}]] == 0, Return["There is no reduced model.  Please generate it at the Model Reduction page"]];
	Clear[V];
	{Tini, redC, redK, redB, inpFun} = readMOR[ID<>"/red.mor", V];
	If[dim > Length[redC], dim = Length[redC]];
	out = "<p> The results are for the dimension of the reduced model equal to "<>ToString[dim]<>".</p><p>Red lines are from ANSYS, green ones from the reduced model.</p>";
	solAt = solveMORAt[time, dim, redC, redK, redB, inpFun];
	plotAll[time, ansys, solAt, V, Tini];
	out
];


